@@ -6,8 +6,8 @@ PDL::AutoLoader - MatLab style AutoLoader for PDL
 =head1 SYNOPSIS
 
  use PDL::AutoLoader;
- $a = func1(...);   # Load file func1.pdl
- $b = func2(...);   # Load file func2.pdl
+ $x = func1(...);   # Load file func1.pdl
+ $y = func2(...);   # Load file func2.pdl
 
  $PDL::AutoLoader::Rescan = 1; # Enable re-scanning
 
@@ -121,7 +121,7 @@ sub PDL::AutoLoader::reloader {
    # Now check functions and reload if changed
 
    my ($file, $old_t);
-   for my $func (keys %PDL::AutoLoader::FileInfo) {
+   for my $func (sort keys %PDL::AutoLoader::FileInfo) {
        ($file, $old_t) = @{ $PDL::AutoLoader::FileInfo{$func} };
        if ( (stat($file))[9]>$old_t ) { # Reload
           print "Reloading $file as file changed...\n" if $PDL::verbose;
