@@ -3,10 +3,14 @@
 #include "pdl.h"      /* Data structure declarations */
 #include "pdlcore.h"  /* Core declarations */
 
-#define REDODIMS(trans) \
-  ((trans)->vtable->redodims \
-    ? (trans)->vtable->redodims \
-    : pdl_default_redodims)(trans)
+#define VTABLE_OR_DEFAULT(trans, func, default_func) \
+  ((trans)->vtable->func \
+    ? (trans)->vtable->func \
+    : pdl_ ## default_func)(trans)
+
+#define REDODIMS(trans) VTABLE_OR_DEFAULT(trans, redodims, redodims_default)
+#define READDATA(trans) VTABLE_OR_DEFAULT(trans, readdata, readdata_affine)
+#define WRITEDATA(trans) VTABLE_OR_DEFAULT(trans, writebackdata, writebackdata_affine)
 
 extern Core PDL;
 
@@ -22,7 +26,7 @@ static int is_parent_of(pdl *it,pdl_trans *trans) {
 
 pdl *pdl_null() {
 	PDL_Indx d[1] = {0};
-	pdl *it = pdl_create(PDL_PERM);
+	pdl *it = pdl_pdlnew();
 	PDL_Anyval zero = { PDL_B, 0 };
 	pdl_makescratchhash(it, zero);
 	pdl_setdims(it,d,1);
@@ -80,25 +84,13 @@ void pdl_allocdata(pdl *it) {
 	it->state |= PDL_ALLOCATED;
 }
 
-/* Wrapper to pdl_create to be stored in the Core struct and exported
-   to external PDL XS modules */
 pdl* pdl_pdlnew() {
-  return pdl_create(PDL_PERM);
-}
-/* Return a new pdl - type is PDL_PERM or PDL_TMP - the latter is fatal error because former uses sv_mortal to be freed
- * when current perl context is left
- */
-pdl* pdl_create(int type) {
      int i;
      pdl* it;
-
-     if(type == PDL_TMP) {croak("PDL internal error. FIX!\n");}
-
      it = (pdl*) malloc(sizeof(pdl));
-     memset(it, 0, sizeof(pdl));
      if (it==NULL)
         croak("Out of Memory\n");
-
+     memset(it, 0, sizeof(pdl));
      it->magicno = PDL_MAGICNO;
      it->state = 0;
      it->datatype = 0;
@@ -108,21 +100,16 @@ pdl* pdl_create(int type) {
      it->datasv = 0;
      it->data = 0;
      it->has_badvalue = 0;
-
      it->dims = it->def_dims;
      it->dimincs = it->def_dimincs;
      it->ndims = 0;
-
      it->nthreadids = 1;
      it->threadids = it->def_threadids;
      it->threadids[0] = 0;
-
      for(i=0; i<PDL_NCHILDREN; i++) {it->children.trans_parent[i]=NULL;}
      it->children.next = NULL;
-
      it->magic = 0;
      it->hdrsv = 0;
-
      PDLDEBUG_f(printf("CREATE %p (size=%zu)\n",(void*)it,sizeof(pdl)));
      return it;
 }
@@ -229,8 +216,7 @@ void pdl__removeparenttrans(pdl *it, pdl_trans *trans, PDL_Indx nth)
 void pdl_destroytransform(pdl_trans *trans,int ensure)
 {
 	PDL_Indx j;
-	pdl *foo;
-	pdl *destbuffer[100];
+	pdl *pdl, *destbuffer[100];
 	int ndest = 0;
 	int ismutual = !(trans->flags & PDL_ITRANS_NONMUTUAL);
 	PDLDEBUG_f(printf("entering pdl_destroytransform %p (ensure %d, ismutual %d)\n",
@@ -248,36 +234,37 @@ void pdl_destroytransform(pdl_trans *trans,int ensure)
 	}
 	if (ismutual) {
 	  for(j=0; j<trans->vtable->nparents; j++) {
-	    foo = trans->pdls[j];
-	    if(!foo) continue;
-	    PDL_CHKMAGIC(foo);
+	    pdl = trans->pdls[j];
+	    if(!pdl) continue;
+	    PDL_CHKMAGIC(pdl);
 	    PDLDEBUG_f(printf("pdl_removectransform(%p): %p %"IND_FLAG"\n",
-	      (void*)trans, (void*)(trans->pdls[j]), j));
-	    pdl__removechildtrans(trans->pdls[j],trans,j,1);
-	    if(!(foo->state & PDL_DESTROYING) && !foo->sv) {
-	      destbuffer[ndest++] = foo;
+	      (void*)trans, (void*)(pdl), j));
+	    pdl__removechildtrans(pdl,trans,j,1);
+	    if(!(pdl->state & PDL_DESTROYING) && !pdl->sv) {
+	      destbuffer[ndest++] = pdl;
 	    }
 	  }
 	  for(; j<trans->vtable->npdls; j++) {
-	    foo = trans->pdls[j];
-	    PDL_CHKMAGIC(foo);
+	    pdl = trans->pdls[j];
+	    PDL_CHKMAGIC(pdl);
 	    PDLDEBUG_f(printf("pdl_removeptransform(%p): %p %"IND_FLAG"\n",
-	      (void*)trans, (void*)(trans->pdls[j]), j));
-	    pdl__removeparenttrans(trans->pdls[j],trans,j);
-	    if(foo->vafftrans) {
-	      PDLDEBUG_f(printf("pdl_removevafft: %p\n", (void*)foo));
-	      pdl_vafftrans_remove(foo);
+	      (void*)trans, (void*)(pdl), j));
+	    pdl__removeparenttrans(pdl,trans,j);
+	    if(pdl->vafftrans) {
+	      PDLDEBUG_f(printf("pdl_removevafft: %p\n", (void*)pdl));
+	      pdl_vafftrans_remove(pdl);
 	    }
-	    if(!(foo->state & PDL_DESTROYING) && !foo->sv) {
-	      destbuffer[ndest++] = foo;
+	    if(!(pdl->state & PDL_DESTROYING) && !pdl->sv) {
+	      destbuffer[ndest++] = pdl;
 	    }
 	  }
 	} else {
 	  PDL_TR_CHKMAGIC(trans);
 	  for(j=trans->vtable->nparents; j<trans->vtable->npdls; j++) {
-	    trans->pdls[j]->state &= ~PDL_NOMYDIMS;
-	    if(trans->pdls[j]->trans_parent == trans)
-	      trans->pdls[j]->trans_parent = 0;
+	    pdl = trans->pdls[j];
+	    pdl->state &= ~PDL_NOMYDIMS;
+	    if(pdl->trans_parent == trans)
+	      pdl->trans_parent = 0;
 	  }
 	}
 	PDL_TR_CHKMAGIC(trans);
@@ -690,11 +677,9 @@ void pdl_setdims_careful(pdl *it)
 /* pdl_get is now vaffine aware */
 PDL_Anyval pdl_get(pdl *it,PDL_Indx *inds) {
         PDL_Indx i;
-        PDL_Indx *incs;
         PDL_Indx offs=PDL_REPROFFS(it);
-        incs = PDL_VAFFOK(it) ? it->vafftrans->incs : it->dimincs;
         for(i=0; i<it->ndims; i++)
-                offs += incs[i] * inds[i];
+                offs += PDL_REPRINC(it,i) * inds[i];
         return pdl_get_offs(PDL_REPRP(it),offs);
 }
 
@@ -838,11 +823,6 @@ void pdl_make_trans_mutual(pdl_trans *trans)
 			trans->pdls[i]->trans_parent = trans;
 		}
 	  }
-#ifdef BARBARBAR /* Not done */
-	  for(i=trans->vtable->nparents; i<trans->vtable->npdls; i++)
-		trans->pdls[i]->state  |=
-		   PDL_PARENTDIMSCHANGED | PDL_PARENTDATACHANGED;
-#endif
 	if(!trans->vtable) {die("INVALID TRANS: has no vtable!\n");}
 	/* now actually perform the transformation, i.e. call
 	   transform's redodims and readdata vtable entries
@@ -867,7 +847,7 @@ void pdl_make_trans_mutual(pdl_trans *trans)
 		pdl_set_trans_childtrans(trans->pdls[i],trans,i);
 	  for(i=trans->vtable->nparents; i<trans->vtable->npdls; i++)
 		pdl_set_trans_parenttrans(trans->pdls[i],trans,i);
-	  if(!(trans->flags & PDL_ITRANS_REVERSIBLE))
+	  if(!(trans->flags & PDL_ITRANS_TWOWAY))
 		trans->flags &= ~PDL_ITRANS_DO_DATAFLOW_B;
 	  for(i=trans->vtable->nparents; i<trans->vtable->npdls; i++) {
 		if(trans->pdls[i]->state & PDL_NOMYDIMS) {
@@ -879,8 +859,8 @@ void pdl_make_trans_mutual(pdl_trans *trans)
   PDLDEBUG_f(printf("make_trans_mutual_exit %p\n",(void*)trans));
 } /* pdl_make_trans_mutual() */
 
-void pdl_default_redodims(pdl_trans *trans) {
-  PDLDEBUG_f(printf("pdl_default_redodims "));
+void pdl_redodims_default(pdl_trans *trans) {
+  PDLDEBUG_f(printf("pdl_redodims_default "));
   PDLDEBUG_f(pdl_dump_trans_fixspace(trans,0));
   PDL_Indx creating[trans->vtable->npdls];
   pdl_transvtable *vtable = trans->vtable;
@@ -980,7 +960,7 @@ void pdl_make_physical(pdl *it) {
 	 *	}
 	 *}
 	 */
-	it->trans_parent->vtable->readdata(it->trans_parent);
+	READDATA(it->trans_parent);
 	it->state &= (~PDL_ANYCHANGED) & (~PDL_OPT_ANY_OK);
 
   mkphys_end:
@@ -1017,7 +997,7 @@ void pdl_children_changesoon(pdl *it, int what)
 	   !(it->trans_parent->flags & PDL_ITRANS_DO_DATAFLOW_B)) {
 		pdl_destroytransform(it->trans_parent,1);
 	} else if(it->trans_parent) {
-		if(!(it->trans_parent->flags & PDL_ITRANS_REVERSIBLE)) {
+		if(!(it->trans_parent->flags & PDL_ITRANS_TWOWAY)) {
 			die("PDL: Internal error: Trying to reverse irreversible trans");
 		}
 		for(i=0; i<it->trans_parent->vtable->nparents; i++)
@@ -1061,11 +1041,8 @@ void pdl_changed(pdl *it, int what, int recursing)
 			pdl_writebackdata_vaffine(it);
 			pdl_changed(it->vafftrans->from,what,0);
 		} else {
-			if(!it->trans_parent->vtable->writebackdata) {
-				die("Internal error: got so close to reversing irrev.");
-			}
 			PDLDEBUG_f(printf("pdl_changed: calling writebackdata from vtable, triggered by pdl %p, using trans %p\n",(void*)it,(void*)(it->trans_parent)));
-			it->trans_parent->vtable->writebackdata(it->trans_parent);
+			WRITEDATA(it->trans_parent);
 			for(i=0; i<it->trans_parent->vtable->nparents; i++) {
 				if((it->trans_parent->vtable->per_pdl_flags[i] &
 				    PDL_TPDL_VAFFINE_OK) &&
@@ -1177,7 +1154,7 @@ void pdl__ensure_trans(pdl_trans *trans,int what)
 			pdl_make_physvaffine(trans->pdls[1]);
 			pdl_readdata_vaffine(trans->pdls[1]);
 		} else {
-			trans->vtable->readdata(trans);
+			READDATA(trans);
 		}
 	}
 	for(j=trans->vtable->nparents; j<trans->vtable->npdls; j++) {
